//Create a namespace if it does not yet exist
Ext.namespace('Zarafa.plugins.spellchecker');

/**
 * @class Zarafa.plugins.spellchecker.SpellcheckerPlugin
 * @extends Zarafa.core.Plugin
 * 
 * A plugin that gives to user the ability to set reminders for e-mail messages.
 */
Zarafa.plugins.spellchecker.SpellcheckerPlugin = Ext.extend(Zarafa.core.Plugin, {

	/**
	 * Initialise the plugin.
	 * @protected
	 */
	initPlugin : function()
	{
		//Call the parent method
		Zarafa.plugins.spellchecker.SpellcheckerPlugin.superclass.initPlugin.apply(this, arguments);

		//Register the language settings widget
		this.registerInsertionPoint('context.settings.category.mail.aftercomposesettings', this.createSettingsWidget, this);

		//Register the provided languages
		this.registerInsertionPoint('common.htmleditor.tinymceconfig', this.createTinyMCEConfig, this);
	},

	/**
	 * Returns an object with the languages that this plugin supports
	 * @return {Object}
	 * @private
	 */
	getSupportedLanguages : function()
	{
		// The plugin needs language pack plugins for any language to work
		// It does not have language support itself
		var supportedLanguages = {};
		
		container.populateInsertionPoint('plugins.spellchecker.supportedlanguages', {scope: this, languages: supportedLanguages});
		
		return supportedLanguages;
	},
	
	/**
	 * Returns the language code for the language that should be used as the default language
	 * @return {String}
	 * @private
	 */
	getDefaultLanguage : function()
	{
		var spellcheckerDefaultLanguage = container.getSettingsModel().get('zarafa/v1/plugins/spellchecker/spellchecker_language');
		var spellcheckerLanguagesSetByUser = this.getUserSelectedLanguages(true);

		if ( !Ext.isEmpty(spellcheckerDefaultLanguage) && spellcheckerLanguagesSetByUser.indexOf(spellcheckerDefaultLanguage) >= 0 ){
			return spellcheckerDefaultLanguage;
		}
		
		// If no default language is selected then do some intelligent stuff to choose a default one
		var language = container.getSettingsModel().get('zarafa/v1/main/language');
		switch ( language.substr(0,2) ){
			case 'de':
				spellcheckerDefaultLanguage = 'de_DE';
				break;
			case 'fr':
				spellcheckerDefaultLanguage = 'fr_FR';
				break;
			case 'it':
				spellcheckerDefaultLanguage = 'it_IT';
				break;
			case 'es':
				spellcheckerDefaultLanguage = 'es_ES';
				break;
			case 'nl':
				spellcheckerDefaultLanguage = 'nl_NL';
				break;
			case 'pt':
				spellcheckerDefaultLanguage = 'pt_PT';
				break;
			case 'ru':
				spellcheckerDefaultLanguage = 'ru_RU';
				break;
			default:
				spellcheckerDefaultLanguage = 'en_US';
		}
		
		if ( spellcheckerLanguagesSetByUser.indexOf(spellcheckerDefaultLanguage) < 0 ){
			// Set the first language as default language
			spellcheckerDefaultLanguage = spellcheckerLanguagesSetByUser.length>0 ? spellcheckerLanguagesSetByUser[0] : '';
		}
		
		container.getSettingsModel().set('zarafa/v1/plugins/spellchecker/spellchecker_language', spellcheckerDefaultLanguage);
		
		return spellcheckerDefaultLanguage;
	},
	
	/**
	 * Returns the languages that the user has selected in the settings for use with the spellchecker
	 * @param {Boolean} languageCodesOnly Set to true if the returned array should only contain the language codes
	 * @private
	 */
	getUserSelectedLanguages : function(languageCodesOnly)
	{
		// Get the languages that are supported by this plugin (and possible language pack plugins) 
		var languageMap = this.getSupportedLanguages();
		
		// Get the languages that the user wants to see in the dropdown and create a string we can pass to tinyMCE in the config
		var spellCheckerLanguagesSetByUser = container.getSettingsModel().get('zarafa/v1/plugins/spellchecker/spellchecker_languages').split(', ');
		
		var spellCheckerLanguages = [];
		Ext.each(spellCheckerLanguagesSetByUser, function(languageKey){
			if ( languageMap.hasOwnProperty(languageKey) ){
				if ( languageCodesOnly ){
					spellCheckerLanguages.push(languageKey);
				}else{
					spellCheckerLanguages.push(languageMap[languageKey] + '=' + languageKey);
				}
			}
		});
		// Sort the array alphabetically (note that translation can change the order)
		spellCheckerLanguages.sort();
		
		return spellCheckerLanguages;
	},
	
	/**
	 * Changes the configuration object for tinyMCE
	 * @param {String} insertionPoint The insertionpoint that the function is populating
	 * @param {Object} options The options object for this insertionpoint
	 * It contains a property called config that holds the configuration for tinyMCE
	 */
	createTinyMCEConfig : function(insertionPoint, options)
	{
		var userSelectedLanguages = this.getUserSelectedLanguages();
		if ( userSelectedLanguages.length === 0 ){
			// When no languages are selected, we don't initialize the spellchecker
			// This way the browsers internal spellchecker can do the work
			return;
		}
		
		// Add the configuration for the tinymce spellchecker plugin
		options.config.spellchecker_rpc_url = 'spellchecker.php';
		options.config.spellchecker_language = this.getDefaultLanguage();
		options.config.spellchecker_languages = userSelectedLanguages.join(', ');
		options.config.browser_spellcheck = false;
		if ( !Ext.isDefined(options.config.external_plugins) ){
			options.config.external_plugins = {};
		}
		// The path is relative from the tinymce folder!
		options.config.external_plugins.spellchecker = '../../plugins/spellchecker/tinymce-plugin/spellchecker/plugin.min.js';
		options.config.toolbar1 = options.config.toolbar1 + ' | spellchecker';
	},
	
	/**
	 * Creates a configuration object for the settings widget used to let the user select the languages he wants
	 * to see in the spellchecker dropdown
	 * @return {Object} Configuration object
	 */
	createSettingsWidget : function()
	{
		return {
			xtype : 'zarafa.settingshtmleditorwidget'
		};
	}
});


//Wait for the DOM and Kopano to be ready and register the plugin
Zarafa.onReady(function() {
	
	//Register the plugin
	container.registerPlugin(new Zarafa.core.PluginMetaData({
		name : 'spellchecker',
		displayName : _('Spellchecker'),
		pluginConstructor : Zarafa.plugins.spellchecker.SpellcheckerPlugin
	}));
});

