Ext.namespace('Zarafa.plugins.spellchecker.settings');

/**
 * @class Zarafa.plugins.spellchecker.settings.SettingsHtmlEditorWidget
 * @extends Zarafa.settings.ui.SettingsWidget
 * @xtype zarafa.settingshtmleditorwidget
 *
 * The {@link Zarafa.settings.ui.SettingsWidget widget} for configuring
 * the html editor.
 */
Zarafa.plugins.spellchecker.settings.SettingsHtmlEditorWidget = Ext.extend(Zarafa.settings.ui.SettingsWidget, {
	
	/**
	 * The name that is used in the settings for the selected languages
	 * @property
	 * @private
	 */
	settingsLanguagesName : 'zarafa/v1/plugins/spellchecker/spellchecker_languages',
	
	/**
	 * The name that is used in the settings for the selected languages
	 * @property
	 * @private
	 */
	settingsDefaultLanguageName : 'zarafa/v1/plugins/spellchecker/spellchecker_language',
	
	/**
	 * @constructor
	 * @param {Object} config Configuration object
	 */
	constructor : function(config)
	{
		config = config || {};

		var spellcheckerPlugin = container.getPluginByName('spellchecker');
		var supportedLanguages = spellcheckerPlugin.getSupportedLanguages();
		
		Ext.applyIf(config, {
			title : _('HTML editor settings'),
			layout : 'form',
			items : []
		});

		if ( Object.keys(supportedLanguages).length > 0 ){
			var selectedLanguages = spellcheckerPlugin.getUserSelectedLanguages();
			var defaultLanguage = spellcheckerPlugin.getDefaultLanguage();
			var data = [];
			Ext.each(selectedLanguages, function(language){
				language = language.split('=');
				data.push([language[1], language[0]]);
			});
			var store = new Ext.data.ArrayStore({
				fields: ['code', 'display'],
				data: data
			});
			
			config.items = [
				{
					xtype : 'displayfield',
					hideLabel : true,
					value : _('Enable the following languages for the spell checker') +':<br><span style="opacity:0.5;">(' + _('Note: When no language is selected the browsers spell checking features will be enabled if available') +')</span>'
				},{
					xtype : 'checkboxgroup',
					name : this.settingsLanguagesName,
					ref : 'spellCheckerLanguagesGroup',
					columns : 1,
					hideLabel : true,
					items: this.createSpellCheckerLanguageOptions(),
					listeners : {
						change : this.onSpellCheckerLanguagesChange,
						scope : this
					}
				},{
					xtype : 'combo',
					name : this.settingsDefaultLanguageName,
					ref : 'spellCheckerDefaultLanguageCombo',
					fieldLabel : _('Default selected language'),
					store : store,
					displayField : 'display',
					valueField : 'code',
					editable : false,
					mode : 'local',
					triggerAction : 'all',
					allowBlank : true,
					autoSelect : true,
					typeAhead : false,
					value : defaultLanguage,
					disabled : selectedLanguages.length === 0 ? true : false,
					validateOnBlur : false,
					invalidClass : '',
					listeners : {
						change : this.onSpellCheckerLanguageChange,
						scope : this
					}
				}
			];
		}else{
			config.items = [{
				xtype : 'displayfield',
				hideLabel : true,
				value : _('No language pack found. Ask your administrator to install language pack plugins to use the spellchecker in the HTML editor.')
			}];
		}

		Zarafa.plugins.spellchecker.settings.SettingsHtmlEditorWidget.superclass.constructor.call(this, config);
	},
	
	/**
	 * Creates the configuration objects for the HTML editor spellchecker language picker
	 * @return {Ext.form.Checkbox[]} Array with configuration object for the checkboxes in the
	 * spellchecker language picker
	 */
	createSpellCheckerLanguageOptions : function()
	{
		var spellcheckerPlugin = container.getPluginByName('spellchecker');
		var supportedLanguages = spellcheckerPlugin.getSupportedLanguages();
		var selectedLanguages = spellcheckerPlugin.getUserSelectedLanguages(true);
		
		var items = [];
		
		Ext.iterate(supportedLanguages, function(languageKey, languageDisplayName){
			items.push({
				xtype : 'checkbox',
				name : 'spellCheckerLanguages',
				inputValue : languageKey,
				boxLabel : languageDisplayName,
				checked : selectedLanguages.indexOf(languageKey)>=0 ? true : false
			});
		});

		//Sort the array alphabetically (note that translation can change the order)
		items.sort(function(a, b){
			return a.boxLabel < b.boxLabel ? -1 : 1;
		});
		
		return items;
	},
	
	/**
	 * Called by the {@link Zarafa.settings.ui.SettingsCategory Category} when
	 * it has been called with {@link zarafa.settings.ui.SettingsCategory#update}.
	 * This is used to load the latest version of the settings from the
	 * {@link Zarafa.settings.SettingsModel} into the UI of this category.
	 * @param {Zarafa.settings.SettingsModel} settingsModel The settings to load
	 */
	update : function(settingsModel)
	{
		if ( !Ext.isDefined(this.spellCheckerLanguagesGroup) ){
			// No language packs were found, so there is nothing to do for us
			return;
		}
		
		this.model = settingsModel;
		
		var spellCheckerLanguagesValue = settingsModel.get(this.settingsName);
		if ( !Ext.isDefined(spellCheckerLanguagesValue) ){
			spellCheckerLanguagesValue = '';
		}
		spellCheckerLanguagesValue = spellCheckerLanguagesValue.split(', ');
		Ext.each(this.spellCheckerLanguagesGroup.items.items, function(checkbox){
			if ( spellCheckerLanguagesValue.indexOf(checkbox.getRawValue())>-1 ){
				checkbox.setValue(true);
			}
		}, this);
	},

	/**
	 * Event handler that handles changes in the spellchecker language settings
	 * @param {Ext.form.CheckboxGroup} checkboxGroup The checkboxgroup that contains the checkboxes
	 * of the spellchecker language picker
	 * @param {Ext.form.Checkbox[]} checkedCheckboxes The checked checkboxes in the group
	 */
	onSpellCheckerLanguagesChange : function(checkboxGroup, checkedCheckboxes)
	{
		if (this.model) {
			var value = [];
			Ext.each(checkedCheckboxes, function(checkbox){
				value.push(checkbox.getRawValue());
			});
			value.sort();
			value = value.join(', ');

			// FIXME: The settings model should be able to detect if
			// a change was applied
			if (this.model.get(checkboxGroup.name) !== value) {
				this.model.set(checkboxGroup.name, value);
			}
		}
		
		// Change the store of the combobox (the default language picker) to contain the selected languages
		var comboBoxValue = this.spellCheckerDefaultLanguageCombo.getValue();
		var comboBoxValueStillChecked = false;
		var data = [];
		Ext.each(checkedCheckboxes, function(checkbox){
			data.push([checkbox.getRawValue(), checkbox.boxLabel]);
			if ( checkbox.getRawValue() === comboBoxValue ){
				comboBoxValueStillChecked = true;
			}
		});
		this.spellCheckerDefaultLanguageCombo.getStore().loadData(data);
		
		if ( checkedCheckboxes.length === 0 ){
			// When no language is selected, we disable the combobox for the default language
			this.spellCheckerDefaultLanguageCombo.setValue('');
			this.spellCheckerDefaultLanguageCombo.disable();
		}else{
			// We make sure the combobox is enabled 
			this.spellCheckerDefaultLanguageCombo.enable();
			if ( !comboBoxValueStillChecked ){
				// Set the selected value to the first value
				this.spellCheckerDefaultLanguageCombo.setValue(checkedCheckboxes[0].getRawValue());
			}
		}
	},

	/**
	 * Event handler that handles changes in the spellchecker default language settings
	 * @param {Ext.form.ComboBox} combo The combobox that contains the default language
	 * of the spellchecker
	 * @param {String} newValue The selected value of the combobox
	 * @param {String} oldValue The previously selected value of the combobox
	 */
	onSpellCheckerLanguageChange : function(combo, newValue, oldValue)
	{
		if (this.model) {
			// FIXME: The settings model should be able to detect if
			// a change was applied
			if (this.model.get(combo.name) !== newValue) {
				this.model.set(combo.name, newValue);
			}
		}
	}
});

Ext.reg('zarafa.settingshtmleditorwidget', Zarafa.plugins.spellchecker.settings.SettingsHtmlEditorWidget);
